@@ -71,6 +71,8 @@ int NetOpened;
 s8* packfile = NULL;
 s8 gamepath[1024]={ '.', '/', 0, 0 };
 s8 save_filename[1024];
+char keys_config_file[MAXPATHLEN] = "/opt/psx4m/keys";
+int bAutosaveOnExit = FALSE;
 
 static void ChangeWorkingDirectory(char *exe)
 {
@@ -233,6 +235,8 @@ accel_move=false;
 		}
 		else if (!strcmp(argv[i], "-mcd1")) 	            sprintf(Config.Mcd1, "%s", argv[++i]);
 		else if (!strcmp(argv[i], "-mcd2")) 	            sprintf(Config.Mcd2, "%s", argv[++i]);
+		else if (!strcmp(argv[i], "-keys")) 				sprintf(keys_config_file, "%s", argv[++i]);
+		else if (!strcmp(argv[i], "-autosave")) 		    bAutosaveOnExit = TRUE;
 		else file = argv[i];
 	}
 	packfile = file;
@@ -240,7 +244,8 @@ accel_move=false;
 	gp2x_init(1000, 16, 11025, 16, 1, 60, 1);
 
 #ifdef MAEMO_CHANGES
-	hildon_init2(&argc, &argv,file,loadst,fullscreen);
+	if (hildon_init2(&argc, &argv,file,loadst,fullscreen))
+		return 1;
 #endif
 
 	if (SysInit() == -1) return 1;
