@@ -6,6 +6,7 @@
 #include <time.h>
 #include <libosso.h>
 #include <dbus/dbus.h>
+#include <hildon/hildon.h>
 
 #include "minimal.h"
 
@@ -20,6 +21,8 @@ GtkWidget* windowG;
 #define F_WIDTH			800
 #define F_HEIGHT		480
 		
+extern char keys_config_file[MAXPATHLEN];
+extern int bAutosaveOnExit;
 int screen_size;
 unsigned short* screenbuffer = NULL;
 
@@ -45,21 +48,8 @@ void hildon_quit()
 
 void show_notification(char* text)
 {
-	DBusConnection* session_bus = (DBusConnection*)osso_get_dbus_connection(osso);
-	DBusMessageIter args;
-	DBusMessage*msg = dbus_message_new_method_call("org.freedesktop.Notifications", 
-												   "/org/freedesktop/Notifications",
-												   "org.freedesktop.Notifications", 
-												   "SystemNoteInfoprint");
-	if (msg) { 
-		dbus_message_iter_init_append(msg, &args);
-		char* param = text;
-		if (dbus_message_iter_append_basic(&args, DBUS_TYPE_STRING, &param)) {
-			dbus_connection_send(session_bus, msg, NULL);
-			dbus_connection_flush(session_bus);
-		}
-		dbus_message_unref(msg);
-	}
+    GtkWidget* banner = hildon_banner_show_information(GTK_WIDGET(window), NULL, text);
+    hildon_banner_set_timeout(HILDON_BANNER(banner), 3000);
 }
 
 static void
@@ -92,15 +82,22 @@ switch(keymap[event->hardware_keycode]){
 	case 9:
 			key = GP2X_START;break;
 	case 10:
-			key = GP2X_VOL_DOWN;break;
-	case 11:
-			key = GP2X_VOL_UP;break;
-	case 12:
 			key = GP2X_L;break;
-	case 13:
+	case 11:
 			key = GP2X_R;break;
+	case 12:
+			key = GP2X_VOL_DOWN;break;
+	case 13:
+			key = GP2X_VOL_UP;break;
 	case 14:
-		running=false;hildon_quit();break;
+		if (bAutosaveOnExit){
+			show_notification("Autosaving");
+			sprintf (cfile, "%s.save.%i", file, 99);
+			SaveState(cfile);
+		}
+		running=false;
+		hildon_quit();
+		break;
 	case 15:
 			if (event->type == GDK_KEY_PRESS){
 			keys |= GP2X_LEFT;
@@ -151,7 +148,6 @@ switch(keymap[event->hardware_keycode]){
 	case 21:
 		if (event->type == GDK_KEY_PRESS){
 			state_slot=(state_slot<9)?state_slot+1:0;
-			//"%.32s-%.9s.%3.3d"
 			sprintf (cfile, "%s.save.%i", file, state_slot);
 			char message[50];
 			sprintf(message,"Savestate slot: %i",state_slot + 1);
@@ -179,7 +175,7 @@ switch(keymap[event->hardware_keycode]){
 }
 
 
-void hildon_init2(int *argc, char ***argv,char *fl,char* lst,bool fscreen)
+int hildon_init2(int *argc, char ***argv,char *fl,char* lst,bool fscreen)
 {
 	fullscreen=fscreen;
 
@@ -196,7 +192,12 @@ void hildon_init2(int *argc, char ***argv,char *fl,char* lst,bool fscreen)
 
 	// read key config
 	FILE* pFile;
-	pFile = fopen("/opt/psx4m/keys", "r"); // assume the file exists and has data
+	pFile = fopen(keys_config_file, "r"); // assume the file exists and has data
+	if (pFile == NULL){
+		fprintf(stderr, "Error opening keys config file %s\n", keys_config_file);
+		return 1;
+	}
+
 	int ch;
 	for (int i=0;i<65536;i++)
 		keymap[i]=164;
@@ -254,6 +255,7 @@ void hildon_init2(int *argc, char ***argv,char *fl,char* lst,bool fscreen)
 	
 	windowG=(GtkWidget*)actor;
 
+	return 0;
 }
 
 void gp2x_change_res(int w, int h)
